#include <iostream>
#include <vector>
#include <sstream>

#include "qfont.h"
#include "qpushbutton.h"
#include "qlayout.h"
#include "qlistbox.h"
#include "qlabel.h"
#include "qmessagebox.h"
#include "qstring.h"

#include "JobSubWindow.h"

#if QT_VERSION >= 300
#include "qtextedit.h"
#else
#include "qmultilineedit.h"
#endif

#if QT_VERSION >= 0x030100
#include "xRSLHighlighter.h"
#endif

#include "CertInfo.h"
#include "Preferences.h"
#include "MdsQuery.h"
#include "Giis.h"
#include "Xrsl.h"
#include "ngui.h"

using namespace std;

JobSubWindow::JobSubWindow( QWidget* parent, const char* name ) : QWidget( parent, name )
{ 
  textFont = new QFont( "System", 11, QFont::Normal );

#if QT_VERSION >= 300
  textEditor = new QTextEdit( this );
  textEditor->setWrapPolicy( QTextEdit::Anywhere ); 
  textEditor->setTextFormat( Qt::PlainText );
# if QT_VERSION >= 0x030100
  xRSLHighlighter* xRSLHighlight = new xRSLHighlighter( textEditor );
# endif
#else
  textEditor = new QMultiLineEdit( this );
  textEditor->setWrapPolicy( QMultiLineEdit::Anywhere ); 
#endif
  textEditor->setFont( *textFont );

  submitButton = new QPushButton( "Submit xRSL", this );
  connect( submitButton, SIGNAL( pressed( ) ), this, SLOT( submit( ) ) );

  updateClusters = new QPushButton( "Update cluster-list", this );
  connect( updateClusters, SIGNAL( pressed( ) ), this, SLOT( updateClusterlist( ) ) );

  clusterView = new QListBox( this );
  clusterView->setSelectionMode( QListBox::Multi );
  connect( clusterView, SIGNAL( doubleClicked( QListBoxItem* ) ), this, SLOT( showInfo( QListBoxItem* ) ) ); 

  clusterSelect = new QPushButton( "Invert chosen clusters", this );
  connect( clusterSelect, SIGNAL( pressed( ) ), clusterView, SLOT( invertSelection( ) ) );

  xRSLLabel = new QLabel( this );
  xRSLLabel->setText( "xRSL window" );

  clusterLabel = new QLabel( this );
  clusterLabel->setText( "Cluster list" );

  screenLayout = new QGridLayout( this, 4, 3, 8 );
  screenLayout->setColStretch( 0, 4 );
  screenLayout->setColStretch( 1, 4 );
  screenLayout->setColStretch( 2, 2 );

  screenLayout->addMultiCellWidget( textEditor, 1, 1, 0, 1 );
  screenLayout->addWidget( xRSLLabel, 0, 0 );
  screenLayout->addWidget( clusterLabel, 0, 2 );
  screenLayout->addWidget( submitButton, 2, 0 );
  screenLayout->addWidget( updateClusters, 3, 2 );
  screenLayout->addWidget( clusterView, 1, 2 );
  screenLayout->addWidget( clusterSelect, 2, 2 );
};

JobSubWindow::~JobSubWindow()
{ 
  delete textEditor;
  delete textFont;
  delete screenLayout;
  delete clusterSelect;
  delete submitButton;
  delete clusterView;
  delete updateClusters;
  delete xRSLLabel;
  delete clusterLabel;
};

void JobSubWindow::updateTab( )
{
  updateClusterlist();
};

void JobSubWindow::setTextArea( QString text )
{
  textEditor->clear();
  textEditor->setText( text );
}

QString JobSubWindow::returnXrsl( )
{
  return textEditor->text();
}

void JobSubWindow::submit( )
{
  QString xrslstring = textEditor->text();
  if( xrslstring.length()==0 ) {
    emit( statusmessage( "No xRSL in window...", 2000 ) );
    return;
  }

  Xrsl xrsl( (const char*)xrslstring );
  if( !xrsl ) {
    emit( statusmessage( "xRSL not valid...", 2000 ) );
    return;
  }

  vector<string> chosenClusters;
  for( int i=0; i<clusterView->numRows(); i++ ) {
    if( clusterView->isSelected( i ) == true ) {
      chosenClusters.push_back( (const char*)clusterView->text( i ) );
    }
  }

  vector<string> dummy;
  string dummystring;
  vector<string> xrsls;
  xrsls.push_back( (const char*)xrslstring );

  // submit job dropping output into stream
  stringstream stream;
  streambuf* sbo = cout.rdbuf(stream.rdbuf());
  streambuf* sbe = cerr.rdbuf(stream.rdbuf());
  int submitstatus = ngsubxx( dummy, xrsls, chosenClusters, dummy, dummy, dummystring, false, false, UNDEFINED, 2, true );
  cerr.rdbuf(sbe);
  cout.rdbuf(sbo);

  if( submitstatus==0 ) {
    emit( statusmessage( "Job successfully submitted...", 2000 ) );
  } else {
    emit( statusmessage( "Job not submitted...", 2000 ) );
  }
}

void JobSubWindow::showInfo( QListBoxItem* item )
{
  vector<Cluster> clusters;
  clusters.push_back( Cluster( (const char*)item->text() ) );

  CertInfo user;
  FindClusterInfo( clusters, Mds::ClusterInfo, user.GetSNx(), true, 40, 0 );

  stringstream outputinfo;
  clusters[0].Print( outputinfo );

  QString caption( "Information about " + item->text() );
  QMessageBox::information( NULL, caption, outputinfo.str().c_str() );
}

void JobSubWindow::updateClusterlist( )
{
  vector<Giis> giislist;
  vector<string> giisurls;
  vector<Cluster> clusters;

  if( GetGiises(giisurls, giislist) ) {
    emit( statusmessage( "Could not get giis-list...", 2000 ) );
    return;
  }

  CertInfo user;
  clusters = FindClusters( giislist, user.GetSNx(), true, 40, 0);

  clusterView->clear();
  for( unsigned int i=0; i<clusters.size(); i++ )
    clusterView->insertItem( QString( clusters[i].GetName().c_str() ) );
  clusterView->sort();
}

void JobSubWindow::getLine( string& s, string& line )
{
  unsigned int i = s.find ('\n');
  if (i == string::npos) return;

  line = s.substr (0, i);
  s = s.substr (i + 1);
}
